<!---+
	Component of utilities for presentations.
		
	@Author Elliott Sprehn
--->
<cfcomponent output="false">
<cfscript>
	import algorithms.sorting.SortAlgorithms;

	/**
	 * Constructor.
	 *
	 * @output false
	 */
	public any function init()
	{
		variables.demo = {};
		
		return this;
	}
	
	/**
	 * Creates an array of random numbers.
	 *
	 * @size   The size of the array to return
	 * @min    Min value of generated numbers
	 * @max    Max value of generated numbers
	 * @sorted If the values should be sorted.
	 *
	 * @output false
	 */
	public array function randArray(required numeric size, numeric min=-1000, numeric max=1000, sorted=false)
	{
		var result = [];
		
		for( var i=1; i <= size; ++i )
			arrayAppend(result,randRange(min,max));
			
		if( sorted )
			arraySort(result,"numeric");
			
		return result;
	}
	
	/**
	 * Creates an array of structures with random properties with number values.
	 *
	 * @size       The size of the array to return
	 * @properties Array property names.
	 * @min        Min value of generated numbers
	 * @max        Max value of generated numbers
	 * @sorted     If the values should be sorted.
	 *
	 * @output false
	 */
	public array function randStructArray(required numeric size, required array properties, 
		numeric min=0, numeric max=1000, comparator=false)
	{
		var result = [];
			
		for( var i=1; i <= size; ++i )
		{
			var item = {};

			for( var prop in properties )
				item[prop] = randRange(min,max);
				
			arrayAppend(result,item);
		}
					
		return new SortAlgorithms().querySort(result,properties);
	}
	
	/**
	 * Comparator for comparing numbers.
	 *
	 * @output false
	 */
	public numeric function numericComparator(required numeric a, required numeric b) 
	{ 
		return a-b; 
	}
	
	/**
	 * Comparator for comparing structs with properties x and y that are numeric.
	 *
	 * @output false
	 */
	public numeric function xyStructComparator(required struct a, required struct b) 
	{
		if( a.x-b.x )
			return a.x-b.x;
		
		return a.y-b.y;
	}
	
	/**
	 * Begins a demo with the provided title.
	 *
	 * @output false
	 */	
	public void function demoStart(string title="")
	{	
		demo.title = title;
		demo.timer = getTickCount();
	}
	
	/**
	 * Ends a demo and optionally prints out vars about it.
	 *
	 * @output true
	 */
	public void function demoEnd(array values=[])
	{
		writeOutput("<div class='demo'>");
		writeOutput("<h4>#htmlEditFormat(demo.title)# (#getTickCount()-demo.timer#ms)</h4>");
		
		for( var value in values )
		{
			if( isArray(value) and arrayLen(value) gt 20 )
			{
				writeOutput("[Array of #arrayLen(value)# values]<br>");
			}
			else
			{
				writeDump(value);
			}
		}
		
		writeOutput("</div>");
		
		structClear(demo);
	}
</cfscript>

	<!---+
		Executes a query of queries against some queries and returns the result.
	--->
	<cffunction name="executeQueryOfQueries" access="public" returntype="query" output="false">
		<cfargument name="sql" type="string" required="true">
		<cfargument name="queries" type="struct" required="true">
				
		<cfset structAppend(local,queries)>
				
		<cfquery name="local.result" dbtype="query">
			#sql#
		</cfquery>
		
		<cfreturn local.result>
	</cffunction>
	
</cfcomponent>