<cfscript>
/**
 * A closure wrapper that either generates a function or takes a function pointer.
 * 
 * @output false
 */
component accessors="true"
{
	property name="function" type="any" setter="false";
	property name="context" type="struct" setter="false";
	
	if( not directoryExists(expandPath("/closures")) )
	{
		try
		{
			directoryCreate(expandPath("/closures"));
		} catch(any err) {}
	}
	
	/**
	 * Constructor.
	 *
	 * @output false
	 */
	public any function init(required any func, required struct context)
	{	
		if( isArray(func) )
		{
			createFunction(func);
		}
		else
		{
			variables.func = func;
		}
		
		variables.context = context;
		
		return this;
	}
	
	/**
	 * Gets the function being used as a closure.
	 *
	 * @output false
	 */
	public any function getFunction()
	{
		return variables.func;
	}
	
	/**
	 * Calls the closure function with it's context.
	 *
	 * @output false
	 */
	public any function call()
	{
		arguments.variables = getContext();
		
		return func(argumentCollection=arguments);
	}
	
	/**
	 * Applies the closure to every element of an array or struct
	 * and replaces the value with the one the closure returns.
	 */
	public any function map(required any values)
	{
		if( isArray(values) )
		{		
			for( var i=1; i lte arrayLen(values); ++i )
				values[i] = func(values[i],getContext());
		}
		else
		{
			for( var key in values )
				values[key] = func(key,values[key],getContext());
		}
		
		return values;
	}
	
	/**
	 * Applies the closure to every element of an array.
	 */
	public any function fold(required array values, required any start)
	{
		var value = start;
		
		for( var i=1; i lte arrayLen(values); ++i )
			value = func(values[i],value,getContext());
		
		return value;
	}
	
	/**
	 * Generates a closure function from a source array. The first
	 * element is a list of argument descriptors, the second is the source.
	 *
	 * @output false
	 */
	private void function createFunction(required array source)
	{
		var key = hash(arrayToList(source));
		source = "<cfscript>private function func(#listAppend(source[1],'variables')#) { #source[2]# }</cfscript>";
		
		if( not structKeyExists(getMetaData(this),key) )
		{
			lock name="Closure(#key#)" timeout="10"
			{
				var path = "/closures/#key#.cfm";
		
				fileWrite(expandPath(path),source);
		
				include path;
			
				getMetaData(this)[key] = func;
				
				fileDelete(expandPath(path));
			}
		}
		
		variables.func = getMetaData(this)[key];
	}
}
</cfscript>